@@ -23,7 +23,7 @@ directory, to make it easily browseable.
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #
-# $Id: repoview.py 376 2008-02-02 18:58:11Z icon $
+# $Id: repoview.py 388 2010-02-19 19:31:04Z icon $
 #
 # Copyright (C) 2005 by Duke University, http://www.duke.edu/
 # Copyright (C) 2006 by McGill University, http://www.mcgill.ca/
@@ -32,13 +32,13 @@ directory, to make it easily browseable.
 #
 #pylint: disable-msg=F0401,W0704
 
-__revision__ = '$Id: repoview.py 376 2008-02-02 18:58:11Z icon $'
+__revision__ = '$Id: repoview.py 388 2010-02-19 19:31:04Z icon $'
 
 import os
 import shutil
 import sys
 import time
-import md5
+import hashlib as md5
 
 from optparse import OptionParser
 from kid      import Template
@@ -68,7 +68,7 @@ RSSKID    = 'rss.kid'
 RSSFILE   = 'latest-feed.xml'
 ISOFORMAT = '%a, %d %b %Y %H:%M:%S %z'
 
-VERSION = '0.6.2'
+VERSION = '0.6.5'
 SUPPORTED_DB_VERSION = 10
 DEFAULT_TEMPLATEDIR = '/usr/share/repoview/templates'
 
@@ -83,7 +83,7 @@ def _mkid(text):
     @rtype:  str
     """
     text = text.replace('/', '.')
-    text = text.replace(' ', '_').lower()
+    text = text.replace(' ', '_')
     return text
 
 def _humansize(bytes):
@@ -307,15 +307,15 @@ class Repoview:
                 comps = os.path.join(self.opts.repodir, href)
         
         if primary is None or dbversion is None:
-            self.say('Sorry, sqlite files not found in the repository. Please '
-                      'rerun createrepo with a -d flag and try again.')
+            self.say('Sorry, sqlite files not found in the repository.\n'
+                     'Please rerun createrepo with a -d flag and try again.\n')
             sys.exit(1)
         
         if int(dbversion) > SUPPORTED_DB_VERSION:
             self.say('Sorry, the db_version in the repository is %s, but '
-                      'repoview only supports versions up to %s. Please check '
-                      'for a newer repoview version.' % (dbversion, 
-                                                         SUPPORTED_DB_VERSION))
+                     'repoview only supports versions up to %s. Please check '
+                     'for a newer repoview version.\n' % (dbversion, 
+                                                          SUPPORTED_DB_VERSION))
             sys.exit(1)
         
         self.say('done\n')
@@ -708,7 +708,7 @@ class Repoview:
         
         for (rpmgroup,) in pcursor.fetchall():  
             qgroup = rpmgroup.replace("'", "''")
-            query = """SELECT name 
+            query = """SELECT DISTINCT name 
                          FROM packages 
                         WHERE lower(rpm_group)='%s'
                           AND %s
@@ -734,19 +734,25 @@ class Repoview:
         @rtype: list
         """
         self.say('Collecting latest packages...')
-        query = """SELECT DISTINCT name, 
-                          version,
-                          release,
-                          time_build 
+        query = """SELECT name
                      FROM packages 
                     WHERE %s
-                 ORDER BY time_build DESC LIMIT %s""" % (self.exclude, limit)
+                    GROUP BY name
+                 ORDER BY MAX(time_build) DESC LIMIT %s""" % (self.exclude, limit)
         pcursor = self.pconn.cursor()
         pcursor.execute(query)
         
         latest = []
-        for (pkgname, version, release, built) in pcursor.fetchall():
-            filename = _mkid(PKGFILE % pkgname)
+        query = """SELECT version, release, time_build
+                     FROM packages
+                    WHERE name = '%s'
+                    ORDER BY time_build DESC LIMIT 1"""
+        for (pkgname,) in pcursor.fetchall():
+            filename = _mkid(PKGFILE % pkgname.replace("'", "''"))
+
+            pcursor.execute(query % pkgname)
+            (version, release, built) = pcursor.fetchone()
+
             latest.append((pkgname, filename, version, release, built))
         
         self.say('done\n')
@@ -760,7 +766,7 @@ class Repoview:
         @rtype:  str
         """
         self.say('Collecting letters...')
-        query = """SELECT DISTINCT substr(upper(name), 0, 1) AS letter 
+        query = """SELECT DISTINCT substr(upper(name), 1, 1) AS letter 
                      FROM packages 
                     WHERE %s
                  ORDER BY letter""" % self.exclude
@@ -782,7 +788,7 @@ class Repoview:
             for (pkgname,) in pcursor.fetchall():
                 pkgnames.append(pkgname)
                 
-            group_filename = _mkid(GRPFILE % rpmgroup)
+            group_filename = _mkid(GRPFILE % rpmgroup).lower()
             letter_group = (rpmgroup, group_filename, description, pkgnames)
             self.letter_groups.append(letter_group)
         self.say('done\n')
